﻿using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Input;
using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Presentation;

namespace UIDemo_Presentation
{
    public sealed class GPIOButtonInputProvider
    {
        public readonly Dispatcher Dispatcher;

        private ButtonPad[] buttons;
        private DispatcherOperationCallback callback;
        private InputProviderSite site;
        private PresentationSource source;

        public GPIOButtonInputProvider(PresentationSource source)
        {
            // memorizziamo la sorgente dell'input
            this.source = source;

            // registriamo questo oggetto come sorgente di "input" e abbiamo
            // indietro un "InputProviderSite", che inoltra le segnalazioni
            // di input all'InputManager, che a sua volta colloca l'input nell'
            // area di "staging"
            site = InputManager.CurrentInputManager.RegisterInputProvider(this);

            // Creiamo un metodo di callback che, a seguito di un "report"
            // di segnalazione di basso livello proveniente da un interrupt,
            // inoltra la notifica all'InputProviderSite
            callback = new DispatcherOperationCallback(delegate(object report)
                {
                    InputReportArgs args = (InputReportArgs)report;
                    return site.ReportInput(args.Device, args.Report);
                });
            Dispatcher = Dispatcher.CurrentDispatcher;

            // Creiamo un provider hardware
            HardwareProvider hwProvider = new HardwareProvider();

            // definiamo i pin relativi ai GPIO collegati ai pulsanti
            // con il default dell'emulatore
            Cpu.Pin pinLeft = Cpu.Pin.GPIO_Pin0;
            Cpu.Pin pinRight = Cpu.Pin.GPIO_Pin1;
            Cpu.Pin pinUp = Cpu.Pin.GPIO_Pin2;
            Cpu.Pin pinSelect = Cpu.Pin.GPIO_Pin3;
            Cpu.Pin pinDown = Cpu.Pin.GPIO_Pin4;

            // Se non siamo sull'emulatore, usiamo la mappatura del
            // HardwareProvider appena definito
            if ((pinLeft = hwProvider.GetButtonPins(Button.VK_LEFT)) ==
                Cpu.Pin.GPIO_NONE)
                pinLeft = Cpu.Pin.GPIO_Pin0;
            else
            {
                pinRight = hwProvider.GetButtonPins(Button.VK_RIGHT);
                pinUp = hwProvider.GetButtonPins(Button.VK_UP);
                pinSelect = hwProvider.GetButtonPins(Button.VK_SELECT);
                pinDown = hwProvider.GetButtonPins(Button.VK_DOWN);
            }

            // Definiamo un insieme di oggetti ButtonPad come array
            // di oggetti che legano il GPIOButtonInputProvider, il codice "virtuale"
            // del pulsante e il pin associato
            ButtonPad[] buttons = new ButtonPad[]
            {
                // Associate the buttons to the pins as discovered or set above
                new ButtonPad(this, Button.VK_LEFT  , pinLeft),
                new ButtonPad(this, Button.VK_RIGHT , pinRight),
                new ButtonPad(this, Button.VK_UP    , pinUp),
                new ButtonPad(this, Button.VK_SELECT, pinSelect),
                new ButtonPad(this, Button.VK_DOWN  , pinDown),
            };

            this.buttons = buttons;
        }

        internal class ButtonPad : IDisposable
        {
            private Button button;          // Codice del pulsante
            private InterruptPort port;     // Porta che genera gli interrupt di notifica
            private GPIOButtonInputProvider sink;
            private ButtonDevice buttonDevice;

            public ButtonPad(GPIOButtonInputProvider sink, Button button,
                Cpu.Pin pin)
            {
                this.sink = sink;
                this.button = button;
                this.buttonDevice = InputManager.CurrentInputManager.ButtonDevice;

                // Se stiamo su un hardware vero e non emulato
                // creiamo una porta interrupt che notifica
                // di entrambi i tipi di variazione
                if (pin != Cpu.Pin.GPIO_NONE)
                {
                    // When this GPIO pin is true, call the Interrupt method.
                    port = new InterruptPort(pin, true,
                        Port.ResistorMode.PullUp,
                        Port.InterruptMode.InterruptEdgeBoth);
                    port.OnInterrupt += new NativeEventHandler(this.Interrupt);
                }
            }

            protected virtual void Dispose(bool disposing)
            {
                if (disposing)
                {
                    if (port != null)
                    {
                        port.Dispose();
                        port = null;
                    }
                }
            }

            public void Dispose()
            {
                Dispose(true);
                GC.SuppressFinalize(this);
            }

            void Interrupt(uint data1, uint data2, DateTime time)
            {
                // Intercetto la varizione dell'ingresso digitale
                // e identifico il tipo di azione effettivamente svolta
                RawButtonActions action = (data2 != 0) ?
                    RawButtonActions.ButtonUp : RawButtonActions.ButtonDown;

                // Creao un nuovo report di segnalazione
                RawButtonInputReport report = new RawButtonInputReport(
                    sink.source, time, button, action);

                // accodo l'evento relativo all'interrupt verso il nostro InputProviderSite
                // utilizzando l'oggetto Dispatcher per poter effettuare
                // la chiamata della callback da un altro thread
                // (il thread dell'interrupt e quello che gestisce la
                // interfaccia utente sono diversi)
                sink.Dispatcher.BeginInvoke(sink.callback, new InputReportArgs(buttonDevice, report));
            }
        }
    }
}
